import { expect, vi} from 'vitest'
import { domTest, flushPromises } from '../test-utils.js'
import '../../../combo/apps/wcs/static/js/combo.wcs-trigger-button.js'

domTest('render button label', async ({appendToDom}) => {
  let dom = appendToDom(`
    <wcs-trigger-button label="Test label"></wcs/trigger-button>
  `)
  const innerButton = dom.querySelector('button')
  expect(innerButton.innerText).toBe('Test label')
})

domTest('show button if trigger-url attribute is set', async ({appendToDom}) => {
  const dom = appendToDom(`
    <wcs-trigger-button label="Test label" trigger-url="https://dummy.org"></wcs/trigger-button>
  `)
  const triggerButton = dom.querySelector('wcs-trigger-button')
  const innerButton = dom.querySelector('button')
  expect(triggerButton.hidden).toBe(false)
  expect(innerButton.disabled).toBe(false)
})

domTest('hide button if trigger-url is not set and unavailable-mode is hide', async ({appendToDom}) => {
  const dom = appendToDom(`
    <wcs-trigger-button unavailable-mode="hide"></wcs/trigger-button>
  `)
  const triggerButton = dom.querySelector('wcs-trigger-button')
  const innerButton = dom.querySelector('button')
  expect(triggerButton.hidden).toBe(true)
  expect(innerButton.disabled).toBe(false)
})

domTest('disable button if trigger-url is not set and unavailable-mode is disable', async ({appendToDom}) => {
  const dom = appendToDom(`
    <wcs-trigger-button unavailable-mode="disable"></wcs/trigger-button>
  `)
  const triggerButton = dom.querySelector('wcs-trigger-button')
  const innerButton = dom.querySelector('button')
  expect(triggerButton.hidden).toBe(false)
  expect(innerButton.disabled).toBe(true)
})

export const triggerTest = domTest.extend({
  clickTrigger: async ({appendToDom}, use) => {
    const load = async (domContent, mockFetch) => {
      const dom = appendToDom(domContent)

      fetch.mockImplementationOnce(mockFetch)

      const innerButton = dom.querySelector('button')
      innerButton.dispatchEvent(new Event('click', {bubbles: true}))
      await flushPromises()

      return dom
    }

    const alertBackup = global.alert
    const confirmBackup = global.confirm
    const fetchBackup = global.fetch

    global.fetch = vi.fn()
    global.alert = vi.fn()
    global.confirm = vi.fn()

    await use(load)

    global.fetch = fetchBackup
    global.confirm = confirmBackup
    global.alert = alertBackup
  },
})

triggerTest('trigger is called on button click', async ({clickTrigger}) => {
  const dom = await clickTrigger(
    `<wcs-trigger-button
      trigger-url="https://trigger.test"
      csrf-token="test-csrf-token"
      proxy-url="https://proxy.test">
    </wcs/trigger-button>`,
    async (url, options) => {
      expect(url).toBe('https://proxy.test')
      expect(options.headers['X-CSRFToken']).toBe('test-csrf-token')
      expect(options.method).toBe('POST')
      expect(JSON.parse(options.body)).toStrictEqual({'trigger-url': 'https://trigger.test'})
      return {ok: true, json: async () => ({}) }
    },
  )

  const innerButton = dom.querySelector('wcs-trigger-button button')
  expect(fetch).toHaveBeenCalledOnce()
  // Button should be disabled after successfull call
  expect(innerButton.disabled).toBe(true)
})

triggerTest('error message is shown on unsuccessfull trigger call', async ({clickTrigger}) => {
  const dom = await clickTrigger(
    '<wcs-trigger-button error-message="Error message"></wcs/trigger-button>',
    async () => ({ok: false, json: async () => ({}) }),
  )

  expect(fetch).toHaveBeenCalledOnce()
  expect(alert).toHaveBeenCalledWith('Error message')
  const innerButton = dom.querySelector('wcs-trigger-button button')
  expect(innerButton.disabled).toBe(false)
})

triggerTest('confirmation message is shown if set', async ({clickTrigger}) => {
  confirm.mockImplementationOnce(() => true)
  await clickTrigger(
    `<wcs-trigger-button confirmation-message="Confirmation">
    </wcs/trigger-button>`,
    async () => ({ok: true, json: async () => ({}) }),
  )

  expect(confirm).toHaveBeenCalledWith('Confirmation')
  expect(fetch).toHaveBeenCalledOnce()
})

triggerTest('trigger is not called if confirmation is dismissed', async ({clickTrigger}) => {
  confirm.mockImplementationOnce(() => false)
  await clickTrigger(
    `<wcs-trigger-button confirmation-message="Confirmation">
    </wcs/trigger-button>`,
  )

  expect(confirm).toHaveBeenCalledWith('Confirmation')
  expect(fetch).not.toHaveBeenCalledOnce()
})
